@@ -997,7 +997,7 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
 
     def _argpairs(self):
         self._valid()
-        
+
         counts = self.counts * (self.counts + 1) >> 1    # N * (N + 1) // 2
 
         offsets = self.counts2offsets(counts)
@@ -1053,7 +1053,130 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         out._parents = parents_comb
         return out
 
+    def _argchoose(self, n, absolute):
+        np = self.numpy
+
+        def root2(a):
+            return np.floor((1+np.sqrt(8*a+1))/2).astype(self.INDEXTYPE)
+
+        def root3(a):
+            out = 2*np.ones(a.shape, dtype=self.INDEXTYPE)
+            mask = a > 0
+            rad = np.power(np.sqrt(3)*np.sqrt(243*a[mask]**2 - 1) + 27*a[mask], 1./3)
+            # 1e-12 to correct rounding error (good to 1000 choose 3)
+            out[mask] = np.floor(np.power(3, -2./3)*rad + np.power(3, -1./3)/rad + 1 + 1e-12)
+            return out
+
+        def root4(a):
+            # good to (at least) 100 choose 4
+            return np.floor((np.sqrt(4*np.sqrt(24*a + 1) + 5) + 3)/2).astype(self.INDEXTYPE)
+
+        if n > 5:
+            # Need to solve general polynomial \prod_{i<=k} (n-i) / k! = a
+            # Not sure if possible, is the Galois group solvable for this class of polynomial?
+            # The first level is a freebie thanks to np.repeat()
+            # I think the next levels should be too, but could not quite make it work...
+            raise NotImplementedError
+        elif n == 5:
+            counts = self.counts*(self.counts - 1)*(self.counts - 2)*(self.counts - 3)*(self.counts - 4)//120
+        elif n == 4:
+            counts = self.counts*(self.counts - 1)*(self.counts - 2)*(self.counts - 3)//24
+        elif n == 3:
+            counts = self.counts*(self.counts - 1)*(self.counts - 2)//6
+        elif n == 2:
+            counts = self.counts*(self.counts - 1)//2
+        elif n <= 1:
+            raise ValueError("Choosing 0 or 1 items is trivial")
+
+        local = np.arange(self.offsets[-1]) - np.repeat(self.offsets[:-1], self.counts)
+        offsets = self.JaggedArray.counts2offsets(counts)
+        indices = np.arange(offsets[-1])
+        parents = self.JaggedArray.offsets2parents(offsets)
+
+        if n == 5:
+            k5 = indices - offsets[parents]
+            i5 = np.repeat(local, local*(local - 1)*(local - 2)*(local - 3)//24)
+            k4 = k5 - i5*(i5 - 1)*(i5 - 2)*(i5 - 3)*(i5 - 4)//120
+            i4 = root4(k4)
+            k3 = k4 - i4*(i4 - 1)*(i4 - 2)*(i4 - 3)//24
+            i3 = root3(k3)
+            k2 = k3 - i3*(i3 - 1)*(i3 - 2)//6
+            i2 = root2(k2)
+            k1 = k2 - i2*(i2 - 1)//2
+            i1 = k1
+            if absolute:
+                starts_parents = self.starts[parents]
+                for idx in [i1, i2, i3, i4, i5]:
+                    idx += starts_parents
+            out = self.JaggedArray.fromoffsets(offsets, self.Table.named("tuple", i1, i2, i3, i4, i5))
+        elif n == 4:
+            k4 = indices - offsets[parents]
+            i4 = np.repeat(local, local*(local - 1)*(local - 2)//6)
+            k3 = k4 - i4*(i4 - 1)*(i4 - 2)*(i4 - 3)//24
+            i3 = root3(k3)
+            k2 = k3 - i3*(i3 - 1)*(i3 - 2)//6
+            i2 = root2(k2)
+            k1 = k2 - i2*(i2 - 1)//2
+            i1 = k1
+            if absolute:
+                starts_parents = self.starts[parents]
+                for idx in [i1, i2, i3, i4]:
+                    idx += starts_parents
+            out = self.JaggedArray.fromoffsets(offsets, self.Table.named("tuple", i1, i2, i3, i4))
+        elif n == 3:
+            k3 = indices - offsets[parents]
+            i3 = np.repeat(local, local*(local - 1)//2)
+            k2 = k3 - i3*(i3 - 1)*(i3 - 2)//6
+            i2 = root2(k2)
+            k1 = k2 - i2*(i2 - 1)//2
+            i1 = k1
+            if absolute:
+                starts_parents = self.starts[parents]
+                for idx in [i1, i2, i3]:
+                    idx += starts_parents
+            out = self.JaggedArray.fromoffsets(offsets, self.Table.named("tuple", i1, i2, i3))
+        elif n == 2:
+            k2 = indices - offsets[parents]
+            i2 = np.repeat(local, local)
+            k1 = k2 - i2*(i2 - 1)//2
+            i1 = k1
+            if absolute:
+                starts_parents = self.starts[parents]
+                for idx in [i1, i2]:
+                    idx += starts_parents
+            out = self.JaggedArray.fromoffsets(offsets, self.Table.named("tuple", i1, i2))
+
+        return out
+
+    def argchoose(self, n):
+        """
+        Return all unique combinations (up to permutation) of n elements,
+        taken without replacement from the indices of the jagged dimension.
+        Combinations are ordered such that those of the first elements precede later ones,
+        e.g. for n=2: (0,1), (0,2), (1,2), (0,3), (1,3), (2,3), (0,4), ...
+        """
+        return self._argchoose(n, absolute=False)
+
+    def choose(self, n):
+        """
+        Return all unique combinations (up to permutation) of n elements,
+        taken without replacement from the contents of the jagged dimension.
+        Combinations are ordered such that those of the first elements precede later ones,
+        e.g. for n=2: (0,1), (0,2), (1,2), (0,3), (1,3), (2,3), (0,4), ...
+        """
+        args = self._argchoose(n, absolute=True)
+        columns = (self.content[args.content[c]] for c in args.columns)
+        out = self.JaggedArray.fromoffsets(args.offsets, self.Table.named("tuple", *columns).flattentuple())
+        return out
+
     def argdistincts(self, nested=False):
+        """
+        Return all unique combinations (up to permutation) of two elements,
+        taken without replacement from the indices of the jagged dimension.
+        Combinations are ordered lexicographically.
+        nested: Return a doubly-jagged array where the first jagged dimension
+        matches the shape of this array
+        """
         out = self._argdistincts(absolute=False)
 
         if nested:
@@ -1062,6 +1185,13 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         return out
 
     def distincts(self, nested=False):
+        """
+        Return all unique combinations (up to permutation) of two elements,
+        taken without replacement from the contents of the jagged dimension.
+        Combinations are ordered lexicographically.
+        nested: Return a doubly-jagged array where the first jagged dimension
+        matches the shape of this array
+        """
         argpairs = self._argdistincts(absolute=True)
         left = argpairs._content["0"]
         right = argpairs._content["1"]
@@ -1395,28 +1525,8 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         newstops.reshape(-1)[flatstarts != flatstops] += 1
         return self.copy(starts=newstarts, stops=newstops, content=flatout)
 
-    @awkward.util.bothmethod
-    def concatenate(isclassmethod, cls_or_self, arrays, axis=0):
-        if isclassmethod: 
-            cls = cls_or_self
-            if not all(isinstance(x, JaggedArray) for x in arrays):
-                raise TypeError("cannot concatenate non-JaggedArrays with JaggedArray.concatenate")
-        else:
-            self = cls_or_self
-            cls = self.__class__
-            if not isinstance(self, JaggedArray) or not all(isinstance(x, JaggedArray) for x in arrays):
-                raise TypeError("cannot concatenate non-JaggedArrays with JaggedArray.concatenate")
-            arrays = (self,) + tuple(arrays)
-
-        for x in arrays:
-            x._valid()
-
-        if axis == 1:
-            return cls._concatenate_axis1(arrays)
-
-        if axis > 1:
-            raise NotImplementedError
-
+    @classmethod
+    def _concatenate_axis0(cls, arrays):
         starts = cls.numpy.concatenate([x._starts for x in arrays])
         stops = cls.numpy.concatenate([x._stops for x in arrays])
         content = cls._util_concatenate([x._content for x in arrays])
@@ -1435,7 +1545,6 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
 
     @classmethod
     def _concatenate_axis1(cls, arrays):
-
         if len(arrays) == 0:
             raise ValueError("at least one array must be provided")   # this can only happen in the classmethod case
         if any(len(a) != len(arrays[0]) for a in arrays):
@@ -1446,10 +1555,7 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         np = cls.numpy
 
         flatarrays = [a.flatten() for a in arrays]
-        n = len(arrays)
-
-        if any(not isinstance(a, np.ndarray) for a in flatarrays):
-            raise NotImplementedError
+        n_arrays = len(arrays)
 
         # the first step is to get the starts and stops for the stacked structure
         counts = np.vstack([a.counts for a in arrays])
@@ -1457,27 +1563,58 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         offsets = cls.counts2offsets(flat_counts)
         starts, stops = offsets[:-1], offsets[1:]
 
-        # find most general type with a tentative sum which implements the right type-promotion,
-        # except for booleans which would get promoted to integers when summing
-        dtype = np.dtype(sum([x[0] for x in flatarrays if len(x) != 0]), False)
-        allbools = not np.any([a.dtype != np.dtype(bool) for a in flatarrays])
-        dtype = np.dtype(bool) if allbools else dtype
-
         n_content = sum([len(a) for a in flatarrays])
+        content_type = type(flatarrays[0])
 
-        # use masks for each of the arrays so we can fill the stacked content array at the right indices
-        content = np.zeros(n_content, dtype=dtype)
-        for i in range(n):
-            working_array = np.zeros(n_content+1, dtype=cls.INDEXTYPE)
-            starts_i = starts[i::n]
-            stops_i = stops[i::n]
+        # get masks for each of the arrays so we can fill the stacked content array at the right indices
+        def get_mask(array_index):
+            working_array = np.zeros(n_content + 1, dtype=cls.INDEXTYPE)
+            starts_i = starts[i::n_arrays]
+            stops_i = stops[i::n_arrays]
             not_empty = starts_i != stops_i
             working_array[starts_i[not_empty]] += 1
             working_array[stops_i[not_empty]] -= 1
             mask = np.array(np.cumsum(working_array)[:-1], dtype=cls.MASKTYPE)
-            content[mask] = flatarrays[i]
+            return mask
 
-        return cls(starts[::n], stops[n-1::n], content)
+        # find most general type with a tentative sum which implements the right type-promotion,
+        # except for booleans which would get promoted to integers when summing
+        def get_dtype(arrays):
+            dtype = np.dtype(sum([x[0] for x in arrays if len(x) != 0]), False)
+            allbools = not np.any([a.dtype != np.dtype(bool) for a in arrays])
+            dtype = np.dtype(bool) if allbools else dtype
+            return dtype
+
+        if content_type == np.ndarray:
+            content = np.zeros(n_content, dtype=get_dtype(flatarrays))
+            for i in range(n_arrays):
+                content[get_mask(i)] = flatarrays[i]
+
+        elif issubclass(type(flatarrays[0]), awkward.array.objects.ObjectArray) and isinstance(flatarrays[0]._content, awkward.array.table.Table):
+            tablecontent = OrderedDict()
+            tables = [a._content for a in flatarrays]
+
+            # make sure all tables have the same columns
+            for i in range(len(tables) - 1):
+                if set(tables[i]._contents) != set(tables[i+1]._contents):
+                    raise ValueError("cannot concatenate Tables with different fields")
+
+            # create empty arrays  for each column with the most general dtype
+            for n in tables[0]._contents:
+                dtype = get_dtype([t[n] for t in tables])
+                tablecontent[n] = np.zeros(n_content, dtype=dtype)
+
+            for i in range(n_arrays):
+                mask = get_mask(i)
+                for n in tables[0]._contents:
+                    tablecontent[n][mask] = tables[i][n]
+
+            content = flatarrays[0].copy(content=awkward.array.table.Table(**tablecontent))
+
+        else:
+            raise NotImplementedError("concatenate with axis=1 is not implemented for " + type(arrays[0]).__name__)
+
+        return arrays[0].__class__(starts[::n_arrays], stops[n_arrays-1::n_arrays], content)
 
     @awkward.util.bothmethod
     def zip(isclassmethod, cls_or_self, columns1={}, *columns2, **columns3):
